/*
 * seismometer 0.1
 * Draw graphs with PSP Motion Kit values.
 * 
 * Copyright (C) 2008  Antoine Jacquet <royale@zerezo.com>
 * http://royale.zerezo.com/psp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <SDL.h>
#include <SDL_rotozoom.h>
#include <SDL_gfxPrimitives.h>

#define WIDTH 480
#define HEIGHT 272
#define BPP 32
#define BLACK SDL_MapRGB(screen->format, 0, 0, 0)
#define WHITE SDL_MapRGB(screen->format, 255, 255, 255)

#ifdef _PSP_FW_VERSION
#include <pspsdk.h>
#include <pspkernel.h>
#define printf pspDebugScreenPrintf
PSP_MODULE_INFO("seismometer",  0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);
#define PSP_UART4_FIFO 0xBE500000
#define PSP_UART4_STAT 0xBE500018
#define PSP_UART_TXFULL  0x20
#define PSP_UART_RXEMPTY 0x10
#endif

SDL_Surface *screen;

int getaxis(int axis)
{
	#ifdef _PSP_FW_VERSION
	pspDebugSioPutchar(axis);
	int i = 0;
	while ((i < 100000) && (_lw(PSP_UART4_STAT) & PSP_UART_RXEMPTY)) i++; // loop until rx buffer full
	return _lw(PSP_UART4_FIFO);
	#else
	return rand() % 5;
	#endif
}

int main(int argc, char *argv[])
{
	int x, ox = 0, dx, odx = 0;
	int y, oy = 0, dy, ody = 0;
	int z, oz = 0, dz, odz = 0;
	SDL_Surface *tmp;
	SDL_Rect dst;
	SDL_Event event;
	
	#ifdef _PSP_FW_VERSION
	pspDebugSioInit();
	pspDebugSioSetBaud(510416);
	#endif
	
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK);
	SDL_JoystickOpen(0);
	screen = SDL_SetVideoMode(WIDTH, HEIGHT, BPP, SDL_HWSURFACE | SDL_ANYFORMAT);
	SDL_FillRect(screen, NULL, BLACK);
	tmp = zoomSurface(screen, 1, 1, 0);
	
	for (;;)
	{
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case SDL_QUIT:
				case SDL_KEYDOWN:
				case SDL_JOYBUTTONDOWN:
					#ifdef _PSP_FW_VERSION
					sceKernelExitGame();
					#else
					exit(0);
					#endif
					break;
			}
		}
		
		dst.x = 1;
		dst.y = 0;
		SDL_FillRect(tmp, NULL, BLACK);
		SDL_BlitSurface(screen, NULL, tmp, &dst);
		
		x = getaxis(0x31);
		y = getaxis(0x32);
		z = getaxis(0x33);
		dx = x-ox;
		dy = y-oy;
		dz = z-oz;
		
		lineColor(tmp, 0, 1*HEIGHT/4+dx, 1, 1*HEIGHT/4+odx, 0xff0000ff);
		lineColor(tmp, 0, 3*HEIGHT/4+dy, 1, 3*HEIGHT/4+ody, 0x00ff00ff);
		lineColor(tmp, 0, 2*HEIGHT/4+dz, 1, 2*HEIGHT/4+odz, 0xffffffff);
		
		ox = x;
		oy = y;
		oz = z;
		odx = dx;
		ody = dy;
		odz = dz;
		
		SDL_BlitSurface(tmp, NULL, screen, NULL);
		SDL_Flip(screen);
	}
}
